@@ -1,14 +1,25 @@
+import re
 from os import environ
 from logging import StreamHandler, INFO
 from functools import wraps
 
+import flask
 from flask import (
-    Flask as Flask_, g, request, Blueprint as Blueprint_, render_template
+    g, request, Blueprint as Blueprint_, render_template,
 )
+from flask.ext.sqlalchemy import SQLAlchemy
+import flask.ext.sqlalchemy
 from flask_debugtoolbar import DebugToolbarExtension
 from pyjade.ext.jinja import PyJadeExtension
 from facebook import parse_signed_request
 
+_underscorer1 = re.compile(r'(.)([A-Z][a-z]+)')
+_underscorer2 = re.compile('([a-z0-9])([A-Z])')
+ 
+def underscored(s):
+    subbed = _underscorer1.sub(r'\1_\2', s)
+    return _underscorer2.sub(r'\1_\2', subbed).lower()
+
 
 class ShortCuts(object):
     def pgroute(self, *args, **kwargs):
@@ -16,7 +27,7 @@ class ShortCuts(object):
         return self.route(*args, **kwargs)
 
 
-class Flask(ShortCuts, Flask_):
+class Flask(ShortCuts, flask.Flask):
     def __init__(self, *args, **kwargs):
         super(Flask, self).__init__(*args, **kwargs)
         # regist jade
@@ -27,6 +38,7 @@ class Flask(ShortCuts, Flask_):
         _config_from_object = self.config.from_object
         def config_from_object(*args, **kwargs):
             _config_from_object(*args, **kwargs)
+            self.config['SQLALCHEMY_ECHO'] = 'SQLALCHEMY_ECHO' in environ
             if self.debug:
                 DebugToolbarExtension(self)
             else:
@@ -42,7 +54,7 @@ class Flask(ShortCuts, Flask_):
 
 
 # extra Features for Facebook
-class FlaskFacebook(Flask_):
+class FlaskFacebook(Flask):
     def __init__(self, *args, **kwargs):
         super(FlaskFacebook, self).__init__(*args, **kwargs)
 
@@ -80,3 +92,37 @@ def templated(template_or_view_func):
             return render_template(template_name, **ctx)
         return decorated_function
     return decorator
+
+def url_for(*args, **kwargs):
+    return flask.url_for(*args, **kwargs)
+
+# looks stupid, but it works...
+models = {}
+_BoundDeclarativeMeta = flask.ext.sqlalchemy._BoundDeclarativeMeta
+class BoundDeclarativeMeta(_BoundDeclarativeMeta):
+    def __init__(cls, name, bases, d):
+        super(BoundDeclarativeMeta, cls).__init__(name, bases, d)
+        if name != 'Model':
+            models[underscored(name)] = cls
+# patch  _BoundDeclarativeMeta
+flask.ext.sqlalchemy._BoundDeclarativeMeta = BoundDeclarativeMeta
+
+class SQLAlchemy(SQLAlchemy):
+    def __init__(self, app, *args, **kwargs):
+        super(SQLAlchemy, self).__init__(app, *args, **kwargs)
+        @app.url_value_preprocessor
+        def model_preprocessor(endpoint, values):
+            if values:
+                for key, id in values.items():
+                    model_name = key[:-3]
+                    model = models.get(model_name, None)
+                    if model:
+                        if id:
+                            if hasattr(id, '__call__'):
+                                value = id()
+                            else:
+                                value = model.query.get_or_404(id)
+                        else:
+                            value = model()
+                        values[key] = value
+
